def solution(lst):
	"""Given a non-empty list of integers, return the sum of all of the odd elements that are in even poitions.
    
	Examples
    solution([5, 8, 7, 1]) = 12
    solution([3, 3, 3, 3, 3]) = 9
    solution([30, 13, 24, 321]) = 0
    """
	
    

print(solution([5, 8, 7, 1]))
print(solution([3, 3, 3, 3, 3]))
print(solution([30, 13, 24, 321]))	